//
//  lph7366.h
//  Graphic LCD module "LPH7366" I/O library
//  Copyright(C)2012-2015 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//		LCD module: LPH7366. (Controller chip is Phillips PCD8544)
//		Interface: SPI+a by software control.
//			MOSI : Any output port (LCD_PIN_SDIN)
//			MISO : Not used
//			SCLK : Any output port (LCD_PIN_SCLK)
//			CS   : Any output port (LCD_PIN_SCE)
//			RESET: Any output port (LCD_PIN_RES) ...optional
//			D/C  : Any output port (LCD_PIN_DC)
//		Screen direction:
//			X: 0=Left .. 83=Right
//			Y: 0=Top .. 47=Bottom
//
//  Device: ATmega328P       IDE: AVR Studio 6.2
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2012.04.08 v0.0  First cording
//    2012.05.03 v0.0  Modify for the TH_SDlogger0.2
//    2015.01.19 v0.3  New API lcd_power(),lcd_contrast(),lcd_draw_image()
//


//------------------------------------------------------------------------------
// Hardware config & Option
//------------------------------------------------------------------------------
#define LCD_PIN_SCLK_ON()		(PORTD |=  _BV(PORTD0))
#define LCD_PIN_SCLK_OFF()		(PORTD &= ~_BV(PORTD0))
#define LCD_PIN_SCLK_INIT()		(DDRD  |=  _BV(DDD0), LCD_PIN_SCLK_OFF())
#define LCD_PIN_SCLK_TERM()		(DDRD  &= ~_BV(DDD0), PORTD &= ~_BV(PORTD0))

#define LCD_PIN_SDIN_ON()		(PORTD |=  _BV(PORTD1))
#define LCD_PIN_SDIN_OFF()		(PORTD &= ~_BV(PORTD1))
#define LCD_PIN_SDIN_INIT()		(DDRD  |=  _BV(DDD1), LCD_PIN_SDIN_OFF())
#define LCD_PIN_SDIN_TERM()		(DDRD  &= ~_BV(DDD1), PORTD &= ~_BV(PORTD1))

#define LCD_PIN_DC_ON()			(PORTD |=  _BV(PORTD3))
#define LCD_PIN_DC_OFF()		(PORTD &= ~_BV(PORTD3))
#define LCD_PIN_DC_INIT()		(DDRD  |=  _BV(DDD3), LCD_PIN_DC_OFF())
#define LCD_PIN_DC_TERM()		(DDRD  &= ~_BV(DDD3), PORTD &= ~_BV(PORTD3))

#define LCD_PIN_SCE_ON()		(PORTD &= ~_BV(PORTD4))		// Neg
#define LCD_PIN_SCE_OFF()		(PORTD |=  _BV(PORTD4))		// Neg
#define LCD_PIN_SCE_INIT()		(DDRD  |=  _BV(DDD4), LCD_PIN_SCE_OFF())
#define LCD_PIN_SCE_TERM()		(DDRD  &= ~_BV(DDD4), PORTD &= ~_BV(PORTD4))

//#define LCD_PIN_RES_ON()		(PORTD &= ~_BV(PORTDx))		// Neg
//#define LCD_PIN_RES_OFF()		(PORTD |=  _BV(PORTDx))		// Neg
//#define LCD_PIN_RES_INIT()	(DDRD  |=  _BV(DDDx), LCD_PIN_RES_OFF())
//#define LCD_PIN_RES_TERM()	(DDRD  &= ~_BV(DDDx), PORTD &= ~_BV(PORTDx))

//#define F_CPU 8000000UL				// Please IDE project configuration option frequency[Hz]
#define LCD_VCD


//------------------------------------------------------------------------------
// Layer 1
//------------------------------------------------------------------------------

extern void lcd_start(void);
extern void lcd_init(void);
extern void lcd_stop(void);
extern void lcd_sendCmdData(unsigned char dc, unsigned char cmd);
#define LCD_DC_COMMAND		0
#define LCD_DC_DATA			1

// LCD module native command code (ref. pcd8544 data sheet)
#define LCD_CMD_FUNC		0b00100000		// Function set
#define LCD_CMD_FUNC_PD		0b00100100		//   Power down mode
#define LCD_CMD_FUNC_V		0b00100010		//   Vertical addressing 
#define LCD_CMD_FUNC_B		0b00100000		//   Use basic instruction set
#define LCD_CMD_FUNC_H		0b00100001		//   Use extended instruction set

#define LCD_CMD_B_BLANK		0b00001000		// Display control
#define LCD_CMD_B_ALLON		0b00001001		//   
#define LCD_CMD_B_NORMAL	0b00001100		//   
#define LCD_CMD_B_INVERSE	0b00001101		//   
#define LCD_CMD_B_YADDR		0b01000000		// Set Y address of RAM  0..5
#define LCD_CMD_B_XADDR		0b10000000		// Set X address of RAM  0..83

#define LCD_CMD_H_TEMP		0b00000100		// Temperature control   0..3
#define LCD_CMD_H_BIAS		0b00010000		// Bias system           0..7
#define LCD_CMD_H_VOP		0b10000000		// Set VOP               0..127


//------------------------------------------------------------------------------
//	Layer 2
//------------------------------------------------------------------------------

//
// Prototype
//
#define     lcd_open()			(lcd_start(), lcd_init())
#define     lcd_close()			lcd_stop()
extern void lcd_contrast(unsigned char level);		// ADD 2015.01.19  level=0..127
extern void	lcd_power(unsigned char mode);			// ADD 2015.01.19
#define LCD_POWER_MODE_NORMAL	0					// ADD 2015.01.19
#define LCD_POWER_MODE_SLEEP	1					// ADD 2015.01.19
#define LCD_POWER_MODE_OFF		2					// ADD 2015.01.19

//extern void lcd_draw_rect(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char color);
//extern void lcd_draw_line(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char line_size, unsigned char color);
extern void lcd_draw_str(unsigned char x, unsigned char y, char *str);
extern void lcd_draw_image(unsigned char x, unsigned char y, unsigned char *bitImage, unsigned char len);	// ADD 2015.01.19

extern void lcd_setViewPort(unsigned char x, unsigned char y, unsigned char width, unsigned char height);
extern void lcd_prString(char *str);
#define     lcd_prCString(s)	lcd_prString(s)
extern void lcd_position(unsigned char column, unsigned char row);
extern void lcd_control(unsigned char command);

//
// General parameter
//
#define LCD_SCREEN_WIDTH	84
#define LCD_SCREEN_HEIGHT	48
#define LCD_FONT_WIDTH		6
#define LCD_FONT_HEIGHT		8
#define LCD_SCREEN_COLUMN	14
#define LCD_SCREEN_ROW		6

//
//	VCD lcd_control() command code
//
#define LCD_DISP_ON                       0x0C
#define LCD_DISP_OFF                      0x08
#define LCD_DISP_BLANK                    0x0A
#define LCD_DISP_CLEAR_HOME               0x01
#define LCD_CURSOR_ON                     0x0E
#define LCD_CURSOR_OFF                    0x0C
#define LCD_CURSOR_WINK                   0x0D
#define LCD_CURSOR_BLINK                  0x0F
#define LCD_CURSOR_SH_LEFT                0x10
#define LCD_CURSOR_SH_RIGHT               0x14
#define LCD_CURSOR_HOME                   0x02
#define LCD_CURSOR_LEFT                   0x04
#define LCD_CURSOR_RIGHT                  0x06
#define LCD_PORT_MASK                     0x7F

